package core;

import java.util.Scanner;
import exception.OutOfBoardException;

/**
 * Cette classe permet d'afficher le plateau et contrler le jeu.
 */
public class Game {

	/**
	 * Le plateau de jeu.
	 */
	private Board board;

	/**
	 * Construit un jeu  partir d'un plateau donn.
	 * 
	 * @param board
	 *            Le plateau de jeu.
	 */
	public Game(Board board) {
		this.board = board;
	}

	/**
	 * Construit par dfaut un jeu avec 2 pions sur un plateau 4x4 cases.
	 */
	public Game() {
		this(new Board(4, 4, 2));
	}

	/**
	 * Dmarre et contrle la boucle de jeu.
	 */
	public void start() {
		Scanner scanner = new Scanner(System.in);
		while (!this.isGameOver()) {
			System.out.println(this);
			int chosenint;
			do {
				System.out.println("Choisissez une direction : ");
				int counter = 0;
				for (Direction d : Direction.values()) {
					System.out.println(counter + ": " + d.name());
					counter++;
				}
				chosenint = scanner.nextInt();
			} while (chosenint >= Direction.values().length || chosenint < 0);
			try {
				System.out
						.println(this.moveNextPawn(Direction.values()[chosenint]));
			} catch (OutOfBoardException e) {
				System.out
						.println("Vous ne pouvez pas aller dans cette direction ! Votre tour est perdu...");
			}
		}
		scanner.close();
	}

	/**
	 * Le jeu est termin s'il reste un seul pion sur le plateau ou si un pion
	 * possde 3 pices d'or ou plus.
	 * 
	 * @return true si le jeu est termin.
	 */
	public boolean isGameOver() {
		return (board.numberOfPawns() == 1) || (board.maxGold() >= 3);
	}

	/**
	 * @return Une chane de caractres dcrivant l'tat courant du jeu : le
	 *         plateau + si le jeu est termin ou non.
	 */
	public String toString() {
		String result = board.toString();
		if (isGameOver())
			result += "\n\n GAME OVER";
		return result;
	}

	/**
	 * Dplace le pion dont c'est le tour dans une direction.
	 * 
	 * @param d
	 *            La direction dans laquelle le pion doit se dplacer.
	 * @return Un message concernant ce qui est arriv.
	 * @throws OutOfBoardException
	 *             si le dplacement est invalide par rapport au plateau.
	 */
	public String moveNextPawn(Direction d) throws OutOfBoardException {
		return this.board.getNextPawn().move(d);
	}

	public static void main(String[] args) {
		new Game().start();
	}

}
